# manages chatbot state
import json


from . import behaviorConfig
from .shellSimulator import WindowsShellSession, LinuxOrMacShellSession

from . import overwriteFileFunction

class ChatBot:
    def __init__(self, user_interface, api_handler, riskAssessmentTool, model, history):
        self.user_interface = user_interface
        self.api_handler = api_handler
        self.conversation_history = [ behaviorConfig.systemPrompt ] + history
        # get the client
        self.client = self.api_handler.get_client()
        # get the model
        self.model = model

        self.tools = behaviorConfig.tools
        self.client = api_handler.get_client()
        self.model = user_interface.get_LLM_model()

        # fixme: get this from somewhere else
        osType = behaviorConfig.get_os_type()
        self.shell = WindowsShellSession(user_interface) if osType == "Windows" else LinuxOrMacShellSession(user_interface)
        self.riskAssessmentTool = riskAssessmentTool


    def conversation_loop(self):
        while True:
            user_message = self.user_interface.get_user_input()
            if user_message == "exit" or user_message == "quit" or user_message == "q":
                break
            self.get_gpt_response(user_message)

    def get_gpt_response(self, prompt):
        self.conversation_history.append({"role": "user", "content": prompt})
        client = self.client
        while True:
            response = client.chat.completions.create(
                model=self.model,
                messages=self.conversation_history,
                tools=self.tools,
                tool_choice="auto"
            )

            # Get the response message
            response_message = response.choices[0].message.content
            if not response_message:
                response_message = ""
            tool_calls = response.choices[0].message.tool_calls
            if not tool_calls:
                self.conversation_history.append({"role": "assistant", "content": response_message})
                if self.user_interface.isInProgess():
                    self.user_interface.inProgressEnd()
            else:
                self.conversation_history.append({"role": "assistant", "content": response_message, "tool_calls": tool_calls})

            if not tool_calls:
                break
            if tool_calls:
                for tool_call in tool_calls:
                    function_name = tool_call.function.name
                    self.user_interface.inProgressStart(tool_call.function.name, tool_call.function.arguments)
                    if function_name == "run_command":
                        # Parse the JSON string into a dictionary
                        arguments = json.loads(tool_call.function.arguments)
                        command = arguments["command"]
                        dangerouslyDisplayFullOutput = arguments["dangerouslyDisplayFullOutput"] if "dangerouslyDisplayFullOutput" in arguments else False
                        self.user_interface.command(command)
                        function_result = ''
                        if (self.riskAssessmentTool.assess_risk(command)):
                            function_result = self.shell.run_command(command, dangerouslyDisplayFullOutput)
                        else:
                            function_result = "USER INTERRUPT: The user denied this command, ask them why and what they would prefer to do instead."
                        self.conversation_history.append({
                            "tool_call_id": tool_call.id,
                            "role": "tool",
                            "name": "run_command",
                            "content": function_result
                        })
                    elif function_name == "overwrite_file":
                        # Parse the JSON string into a dictionary
                        arguments = json.loads(tool_call.function.arguments)
                        filepath = arguments["filepath"]
                        content = arguments["content"]
                        ## error handling
                        try:
                            overwriteFileFunction.write_content_to_file(filepath, content)
                            self.conversation_history.append({
                                "tool_call_id": tool_call.id,
                                "role": "tool",
                                "name": "overwrite_file",
                                "content": "File successfully overwritten"
                            })
                        except ValueError as e:
                            self.conversation_history.append({
                                "tool_call_id": tool_call.id,
                                "role": "tool",
                                "name": "overwrite_file",
                                "content": str(e)
                            })

                        
                    else:
                        self.conversation_history.append({
                            "tool_call_id": tool_call.id,
                            "role": "tool",
                            "name": function_name,
                            "content": "Tool not implemented"
                        })
        self.user_interface.chatBotMessage(response_message)
        



import configparser
import os
from openai import OpenAI

# Determine the appropriate config path based on the operating system
def get_config_path():
    return os.path.expanduser('~/.gptautocli.config')

config_path = get_config_path()
config = configparser.ConfigParser()

class ConfigHandler:
    def __init__(self, user_interface):
        self.user_interface = user_interface
        self.setup_api_key()
        self.setup_command_risk()

    # allows the user to decide what threshold of risk a command needs before requiring confirmation
    def setup_command_risk(self):
        if 'Command_Risk' not in config['DEFAULT']:
            prompt_message = (
                "A separate AI model evaluates each command generated by gptautocli to determine how risky it is on a scale of 1-5. \n"
                "For example 'ls' would be a 1, while 'rm -rf *' would be a 5. \n"
                "For a more in depth explanation of each risk level and how this system works, see the README (https://github.com/BSchoolland/gptautocli)\n"
                "Entering 0 will require confirmation for all commands, while 6 will run all commands without confirmation. \n"
                "Please enter the threshold of risk that a command must be below to be executed without confirmation (1-5)"
            )
            command_risk = self.user_interface.dialog(prompt_message)
            if command_risk.isdigit() and 0 <= int(command_risk) <= 6:
                config['DEFAULT']['Command_Risk'] = command_risk
                with open(config_path, 'w') as configfile:
                    config.write(configfile)
                self.user_interface.info("Command risk threshold saved successfully.")
            else:
                self.user_interface.error("Invalid command risk threshold entered. Please enter a number between 0 and 6.")
                exit(1)
        else:
            config.read(config_path)
        

    def setup_api_key(self):
        """
        Setup the OpenAI API key
        """
        # Read the existing configuration if it exists
        if os.path.exists(config_path):
            config.read(config_path)
        
        if 'OpenAI_API_Key' not in config['DEFAULT']:
            # Determine the prompt message based on the operating system
            
            prompt_message = (
                "gptautocli requires a valid OpenAI API key (which will not be shared with anyone) to function. \n"
                "Please add your key to ~/.gptautocli.config and restart the program or enter it here"
            )
            
            api_key = self.user_interface.dialog(prompt_message)
            
            # Confirm that the key works
            client = OpenAI(api_key=api_key)
            try:
                if self.test_client(client):
                    self.user_interface.info("API key verified successfully.")
                else:
                    self.user_interface.error("You entered: " + api_key)
                    self.user_interface.error("API key verification failed. Please check your API key and try again.")
                    exit(1)
            except Exception as e:
                self.user_interface.error("You entered: " + api_key)
                self.user_interface.error("API key verification failed. Please check your API key and try again.")
                print(f"Error: {type(e).__name__}, {str(e)}")
                exit(1)

            # Update the configuration with the new API key
            config['DEFAULT']['OpenAI_API_Key'] = api_key
            with open(config_path, 'w') as configfile:
                config.write(configfile)
            self.user_interface.info("API key saved successfully.")
        else:
            config.read(config_path)
    
    def test_client(self, client):
        """
        Test the OpenAI API client by sending a simple prompt

        Parameters
        ----------
        client : OpenAI
            The OpenAI API client

        Returns
        -------
        bool
            True if the client is working, False otherwise
        """
        # Add user input to the conversation history
        conversation_history = [{"role": "user", "content": "Hi"}]

        response = client.chat.completions.create(
            model="gpt-4o-mini",
            messages=conversation_history
        )

        # Get the response message
        response_message = response.choices[0].message.content

        # Add the response to the conversation history
        conversation_history.append({"role": "assistant", "content": response_message})

        return bool(response_message)


import configparser
import os
import platform
from openai import OpenAI
from . import apiHandler

from .behaviorConfig import riskAssessmentPrompt

# Determine the appropriate config path based on the operating system
def get_config_path():
    if platform.system() == 'Windows':
        return os.path.join(os.getenv('APPDATA'), 'gptautocli', 'config.ini')
    else:
        return os.path.expanduser('~/.gptautocli.config')

config_path = get_config_path()
config = configparser.ConfigParser()

class RiskAssessment:
    def __init__(self, user_interface, api_handler, risk_tolerance = -1):
        self.user_interface = user_interface
        self.api_handler = api_handler
        if risk_tolerance != -1:
            self.risk_tolerance = risk_tolerance
        else:
            self.risk_tolerance = int(self.get_risk_tolerance())
        print('Risk tolerance:', self.risk_tolerance)
        self.api_handler = apiHandler.ApiHandler(user_interface)

    def assess_risk(self, command):
        if self.risk_tolerance == 6:
            return True
        else:
            # get the risk score 
            messages = [
                riskAssessmentPrompt,
                {
                    "role": "user", "content": command
                }
            ]
            response = self.api_handler.get_client().chat.completions.create(
                model='gpt-4o-mini',
                messages=messages,
            )

            # Get the response message
            response_message = response.choices[0].message.content
            risk_score = -1
            # get the highest integer in the response message
            for char in response_message:
                if char.isdigit() and int(char) > risk_score:
                    risk_score = int(char)
            # catch case where risk assessment AI does not return a risk score
            if risk_score == -1:
                risk_score = 3 # default to medium risk
            
            if risk_score < self.risk_tolerance:
                return True
            else:
                return self.user_interface.riskConfirmation(command, risk_score)
    
    def get_risk_tolerance(self):
        config.read(config_path)
        if 'command_risk' in config['DEFAULT']:
            return config['DEFAULT']['command_risk']
        else:
            return 0

    


# class for saving and loading history to a json file
import json
import os

class HistoryManager:
    def __init__(self):
        pass
        
    def load_chat_history(self):
        if os.path.exists('history.json'):
            with open('history.json', 'r') as file:
                return []
        else:
            return []
    
    def save_chat_history(self, chat_history):
        pass # FIXME: implement this
    
        # removed because tool calls cause errors when converting to json
        # with open('history.json', 'w') as file:
        #     json.dump(chat_history, file)
            



import configparser
import os
import platform
from openai import OpenAI

# Determine the appropriate config path based on the operating system
def get_config_path():
    return os.path.expanduser('~/.gptautocli.config')

config_path = get_config_path()
config = configparser.ConfigParser()

class ApiHandler:
    def __init__(self, user_interface):
        self.user_interface = user_interface
        self.client = OpenAI(api_key=self.get_api_key())

    def get_client(self):
        return self.client
    
    def get_api_key(self):
        config.read(config_path)
        if 'openai_api_key' in config['DEFAULT']:
            return config['DEFAULT']['openai_api_key']
        else:
            self.user_interface.error("API key not found. Please run the setup process again.")
            exit(1)

    


import os
import subprocess
import select
import time
import sys
import selectors

import sys
from . import behaviorConfig

if behaviorConfig.get_os_type() == "Windows":
    import msvcrt  # Only import msvcrt if running on Windows

# parent class for all operating systems
class ShellSession:
    def __init__(self, userInterface=None):
        self.userInterface = userInterface
        self.command_counter = 0
    # same for all operating systems
    def is_command_allowed(self, command):
        # list of disallowed commands: nano, vi, vim FIXME: add windows and mac commands
        disallowed_commands = ["nano", "vi", "vim"]
        for disallowed_command in disallowed_commands:
            if command.startswith(disallowed_command):
                return f"TERMINAL ERROR: Command '{disallowed_command}' is not allowed. Please try using an alternative command ex: 'echo instead of nano'."
        # make sure the command does not include ``` bash or ```shell
        return "Yes"
    
    # to be implemented by the child classes
    def run_command(self, command, dangerouslyDisplayFullOutput=False):
        pass
    def close(self):
        pass
    def getCurrentDirectory(self):
        pass

class LinuxOrMacShellSession(ShellSession):
    def __init__(self, userInterface=None):
        super().__init__(userInterface)
        import pty
        master, slave = pty.openpty()
        self.process = subprocess.Popen(
            ['/bin/bash'],
            stdin=slave,
            stdout=slave,
            stderr=subprocess.STDOUT,
            close_fds=True,
            preexec_fn=os.setsid
        )
        self.master_fd = master
        os.close(slave)
    

    def run_command(self, command, dangerouslyDisplayFullOutput=False):
        # check if the command is allowed
        if self.is_command_allowed(command) != "Yes":
            return self.is_command_allowed(command)
            
        self.command_counter += 1  # Increment command counter
        end_tag = f"COMMAND_DONE_TAG{self.command_counter}"
        # Send command
        os.write(self.master_fd, (" " + command + "; echo " + end_tag + "\n").encode('utf-8'))
        Done = False
        first = True
        output = []

        while not Done:
            # Wait for input from either the process or stdin, with a timeout
            r, _, _ = select.select([self.master_fd, sys.stdin], [], [], 0.5)
            for ready_input in r:
                if ready_input == self.master_fd:
                    response = os.read(self.master_fd, 1024).decode('utf-8')
                    # Break the command up into lines
                    responses = response.split("\r\n")
                    for response in responses:
                        
                        if end_tag in response and command not in response:
                            # Command output finished
                            Done = True
                            break
                        if first:
                            # Skip the first line which is the prompt
                            first = False
                            continue
                        elif command + "; echo " + end_tag in response:
                            # Skip the command echo
                            continue
                        if self.userInterface:
                            self.userInterface.commandResult(response)
                        output.append(response)
                        

                elif ready_input == sys.stdin:
                    # Read from stdin (user input)
                    userInput = input() 
                    if userInput == "":
                        userInput = "\n" # if the user just presses enter, send a newline
                    if userInput == "exit" or userInput == "quit" or userInput == "q":
                        print("User interruption detected.")
                        Done = True
                        output.append("User ended the process. Exiting...")
                        # stop the process
                        os.write(self.master_fd, b"\x03")
                    else:
                        # write the input to the process
                        os.write(self.master_fd, (userInput + "\n").encode('utf-8'))

            # Check if the process has terminated
            if self.process.poll() is not None:
                break

        result = '\n'.join(output).strip()
        # limit the output to 1000 characters
        if len(result) > 1000 and not dangerouslyDisplayFullOutput:
            x = str(len(result) - 1000)
            result = result[:500] + "... " + x + " characters truncated to save tokens. ..." + result[-500:]
        return result

    def close(self):
        try:
            os.write(self.master_fd, b"exit\n")
            time.sleep(2)  # Give time for the exit command to process
        finally:
            os.close(self.master_fd)
            self.process.wait()
    def getCurrentDirectory(self):
        return self.run_command("pwd")


class WindowsShellSession(ShellSession):
    def __init__(self, userInterface=None):
        super().__init__(userInterface)
        # Create a persistent process that stays alive between commands
        self.process = subprocess.Popen(
            'cmd.exe',
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,
            bufsize=1,  # Line buffered
            universal_newlines=True
        )

    def run_command(self, command, dangerouslyDisplayFullOutput=False):
        # Check if the command is allowed
        command_status = self.is_command_allowed(command)
        if command_status != "Yes":
            return command_status

        self.command_counter += 1
        end_tag = f"COMMAND_DONE_TAG{self.command_counter}"
        
        # Send the command
        try:
            self.process.stdin.write(f"{command}\n")
            self.process.stdin.write(f"echo {end_tag}\n")
            self.process.stdin.flush()
        except IOError:
            return "Error: Failed to send command to process"

        output = []
        command_output_started = False
        Done = False

        while not Done:
            # Use a small timeout to avoid busy waiting
            try:
                line = self.process.stdout.readline()
                if not line:
                    Done = True
                    break
                    
                # Skip command echo and empty lines at the start
                if not command_output_started:
                    if command in line or not line.strip():
                        continue
                    command_output_started = True
                    
                if end_tag in line:
                    Done = True
                    break
                    
                if self.userInterface:
                    self.userInterface.commandResult(line.rstrip())
                output.append(line)

            except IOError:
                Done = True
                break

            # Check for user input (non-blocking)
            if msvcrt.kbhit():
                char = msvcrt.getwche()  # Use getwche for better Unicode support
                
                # Handle special cases
                if char == '\r':  # Enter key
                    print()  # New line after enter
                    self.process.stdin.write('\n')
                    self.process.stdin.flush()
                elif char == '\x03':  # Ctrl+C
                    print("^C")
                    self.process.stdin.write('\x03')
                    self.process.stdin.flush()
                    Done = True
                    output.append("User interrupted the process")
                elif char == '\x1a':  # Ctrl+Z
                    print("^Z")
                    self.process.stdin.write('\x1a')
                    self.process.stdin.flush()
                else:
                    # Send regular character input
                    try:
                        self.process.stdin.write(char)
                        self.process.stdin.flush()
                    except IOError:
                        Done = True
                        break

            # Check if process has terminated
            if self.process.poll() is not None:
                Done = True
                break

        result = ''.join(output).strip()
        # limit the output to 1000 characters
        if len(result) > 1000 and not dangerouslyDisplayFullOutput:
            x = str(len(result) - 1000)
            result = result[:500] + "... " + x + " characters truncated to save tokens. ..." + result[-500:]
        # Continue reading while the subprocess is running
        while True:
            line = self.process.stdout.readline()
            if not line:
                break  # No more output
            if end_tag in line:
                break  # Command output finished
            output.append(line)
        
        result = ''.join(output)
        
        return result

    def close(self):
        if self.process:
            try:
                self.process.stdin.write("exit\n")
                self.process.stdin.flush()
                time.sleep(1)  # Give time for the exit command to process
            except:
                pass  # Ignore errors during cleanup
            finally:
                self.process.terminate()
                self.process.wait()
    
    def getCurrentDirectory(self):
        return self.run_command("cd")




if __name__ == '__main__':
    print("<--BEGIN AUTOMATED TERMINAL SESSION-->")
    shell = LinuxOrMacShellSession()
    result = shell.run_command("ls -l")
    print("<--END AUTOMATED TERMINAL SESSION-->")
    print('the result is: ', result)
    shell.close()


# This is the entry point for the AI terminal assistant program.  It outlines the main program flow and calls other files at a high level.

# Package execution
from . import apiHandler
from . import userInterface
from . import historyManager
from . import chatBot
from . import configHandler
from . import riskAssessment

import argparse
import sys

def parse_args():
    """Parse command line arguments for gptautocli"""
    parser = argparse.ArgumentParser(
        description="gptautocli - Provides a conversational interface to the terminal, allowing users to run commands and perform tasks using natural language.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
    gptautocli                            # Start a new chat with default settings
    gptautocli -m your-model-name         # Start a new chat with the model of your choice (e.g. gpt-4, gpt-3.5-turbo, etc.)
    gptautocli --risk-level 3             # Set the risk tolerance to 3
    gptautocli -h                         # Display this help message
    gptautocli -v                         # Display the version number
        """) 
    
    parser.add_argument("-v", "--version", action="store_true", help="Display the version number")
    parser.add_argument("-m", "--model", type=str, help="Specify the model to use for the chat")
    parser.add_argument("-r", "--risk-level", type=int, help="Specify the risk tolerance level for commands generated by the AI")
    return parser.parse_args()
                                    

def main():
    """Main program flow"""
    # parse command line arguments
    args = parse_args()
    if args.version:
        # Read the version from the VERSION file
        with open("VERSION") as version_file:
            version = version_file.read().strip()
        print(f"gptautocli version {version}")
        sys.exit()
    model_name = "gpt-4o"

    if args.model:
        model_name = args.model
    risk_level = -1
    if args.risk_level is not None:
        risk_level = args.risk_level
    # initialize the UI
    user_interface = userInterface.UserInterface()
    # initialize config
    configHandler.ConfigHandler(user_interface)
    # initialize the API handler
    api_handler = apiHandler.ApiHandler(user_interface)
    # risk assessment tool
    riskAssessmentTool = riskAssessment.RiskAssessment(user_interface, api_handler, risk_level)
    risk_level = riskAssessmentTool.risk_tolerance
    # welcome the user
    user_interface.welcome(model_name, risk_level)
    # initialize the history manager
    history_manager = historyManager.HistoryManager()
    # load chat history
    all_chat_history = history_manager.load_chat_history()
    # choose either a new chat or a previous chat
    history = user_interface.choose_chat_history(all_chat_history)
    
    # initialize the agent manager
    agent_manager = chatBot.ChatBot(user_interface, api_handler, riskAssessmentTool, model_name, history)

    # begin the conversation loop
    agent_manager.conversation_loop()
    # close the shell
    agent_manager.shell.close()
    # once the conversation is over, save the chat history to a new file
    history_manager.save_chat_history(agent_manager.conversation_history)


if __name__ == '__main__':
    try:
        # attempt to run the main program
        main()
    except Exception as e:
        print("An error occurred: ", e)
        input("Press enter to exit.")



import os

def write_content_to_file(filepath: str, content: str) -> None:
    """
    Writes content to a file using platform-specific methods.
    
    Args:
        filepath: The path to the file to write to
        content: The content to write to the file
    
    Raises:
        ValueError: If the provided filepath is not an absolute path.
    """
    
    # Check if the provided path is absolute
    if not os.path.isabs(filepath):
        raise ValueError("The provided filepath must be an absolute path.")
    
    # Construct the absolute file path
    file_path = os.path.abspath(filepath)
    
    # Ensure the directory exists
    directory = os.path.dirname(file_path)
    if not os.path.exists(directory):
        os.makedirs(directory)
    
    # Write content to the file
    with open(file_path, 'w', encoding='utf-8') as file:
        file.write(content)
    



# the system prompt that tells the AI what it is supposed to do

from .getTerminal import get_os_type, get_terminal_type

import os
osType = get_os_type()
terminalType = get_terminal_type()
currentDir = os.getcwd()
from .shellSimulator import LinuxOrMacShellSession, WindowsShellSession

directoryContents = ""
if osType == "Windows":
    shellSession = WindowsShellSession()
    directoryContents = shellSession.run_command("dir")
    shellSession.close()
else:
    shellSession = LinuxOrMacShellSession()
    directoryContents = shellSession.run_command("ls -a")
    shellSession.close()

# overview of how the chatbot should behave
systemPrompt = {"role": "system", "content": """You are an intelligent and somewhat autonomous AI system called 'gptautocli' running on a """ + osType + """ system with a """ + terminalType + """ terminal.  You are capable of running most commands in the terminal using the provided tool.  The one limitation is that you cannot run commands like `nano` or `vim` that require user input or a GUI.  If you need to create a file, use `echo` instead.  You can also evaluate mathematical expressions using the provided tool.  Before starting on a task, please create a detailed plan of how you will accomplish the task, and ask the user for confirmation before executing the series of commands.   
                
Context: You started in the directory """ + currentDir + """which had the following files and directories: 
""" + directoryContents + """

Example of how a conversation might go:
User: Can you create a node server and html file, with a simple api that returns "Hello, World!"?
You: Sure, I can help with that. Here's the plan to create a Node.js server and an HTML file with a simple API that returns "Hello, World!":
    1. Check if Node.js is installed, if not, install it.
    2. Create a directory for the Node.js server.
    3. Initialize a Node.js project in the directory.
    4. Install the Express and CORS packages, which we will need for the api
    5. Create a server.js file for the Node.js server.
    6. Create an index.html file with a button that will trigger the api call.
    7. Start the Node.js server.

Would you like me to proceed with this plan?

User: Yes, that sounds good. Please go ahead.

You: run_command(node --version)
Tool: node: command not found
You: run_command(sudo apt install nodejs npm -y)
You: run_command(node --version)
Tool: v14.17.0
You: run_command(mkdir node-server)
You: run_command(cd node-server)
You: run_command(npm init -y)
You: run_command(npm install express)
You: run_command(npm install cors)
You: overwrite_file("""+currentDir+"""/node-server/server.js
, `
const express = require("express");
const cors = require("cors");

const app = express();
app.use(cors({
    origin: "*"
}));

// Serve static files from current directory
app.use(express.static('./'));

const port = 3000;

app.get('/api', (req, res) => {
    console.log("Hello world!");
    res.json({
        message: "Hello World!"
    });
});

app.listen(port, () => {
    console.log(`Server is running http://localhost:${port}`);
});
`)
You: overwrite_file("""+currentDir+"""/node-server/index.html
, `
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Document</title>
</head>
<body>
    <button onclick="apiCall(event)">Make API call</button>
</body>

<script>
    function apiCall(event) {
        fetch('http://localhost:3000/api')
            .then(response => response.json())
            .then(data => {
                alert(data.message);
            })
            .catch(error => {
                alert('Error making API call: ' + error.message);
            });
    }
</script>
</html>
`)
You: run_command(node server.js)
Tool: Server is running on http://localhost:3000
You: The Node.js server is now running. You can access the webpage at http://localhost:3000/index.html in your web browser. When you click the button, it will make an API call to the server and display the message it receives from the server (Hello World!) in an alert.  Is there anything else you would like me to do?
"""
}

tools = [
    {
        "type": "function",
        "function": {
            "name": "run_command",
            "description": "Powerful function that will execute nearly any command in the terminal",
            "parameters": {
                "type": "object",
                "properties": {
                    "command": {
                        "type": "string",
                        "description": "The command to run in the terminal",
                    },
                    "dangerouslyDisplayFullOutput": {
                        "type": "boolean",
                        "description": "If true, the full output of the command will be displayed. This is useful for reading code and documentation. Do NOT use this on commands like npm i or other commands that output a lot of data.",
                    }
                },
                "required": ["command"],
            },
        },
    },
    {
        "type": "function",
        "function": {
            "name": "overwrite_file",
            "description": "Overwrite a file with new content, useful for adding code or other content to a file",
            "parameters": {
                "type": "object",
                "properties": {
                    "filepath": {
                        "type": "string",
                        "description": "The full path of the file to create/overwrite",
                    },
                    "content": {
                        "type": "string",
                        "description": "The new content to write to the file",
                    },
                },
                "required": ["file", "content"],
            },
        }
    }
]


# config for the smaller AI that assesses the risk of each command
riskAssessmentPrompt = {"role": "system", "content": """

1 - Read Only: Commands that simply read data without modifying anything at all
Example: ls -l - Lists files in a directory, changing nothing

2 - Safe: Commands that can write data but can not do any accidental damage
Example: touch myfile.txt - creates a file if it does not exist, but will not overwrite existing files

3 - Low Risk: Commands that alter files or locations, risky as it causes a change to the system.
Example: echo "text" >> myfile.txt - adds some data to the file

4 - High Risk: Commands that can modify data or cause other problems, leading to some data loss or serious inconvenience if used wrongly.
Example: rm myfile.txt - Deletes a file, leading to data loss if the wrong file is targeted

5 - Critical Risk (Accident-Prone): Commands that can cause severe damage or data loss if accidentally misused, often with no recovery option.
Example: rm -rf /Projects - Deletes all files in a likely important directory, leading to data loss

The user is going to provide a command.  Your output should be in this exact format:

[[Risk Assessment: min-max]]

With min and max being the minimum and maximum risk levels of the command.  
"""}

# riskAssessmentTool = {
#     "type": "function",
#     "function": {
#         "name": "riskAssessment",
#         "description": "Call this function to provide a risk assessment of a command.",
#         "parameters": {
#             "type": "object",
#             "properties": {
#                 "minRisk": {
#                     "type": "integer",
#                     "description": "The minimum risk level of the command (1-5)",
#                 },
#                 "maxRisk": {
#                     "type": "integer",
#                     "description": "The maximum risk level of the command (1-5)",
#                 },
#             },
#             "required": ["minRisk", "maxRisk"],
#         },
#     },
# }


# Handles user input and provides output


from colorama import Fore, Style, init
import getpass
import os
from gptautocli.getTerminal import get_os_type

init() # ensure colorama works on windows
    
class UserInterface:
    def __init__(self):
        self.model = None
        self.inProgress = False

    def welcome(self, model, risk_tolerance):
        osType = get_os_type()

        # Disclaimer
        current_dir = os.getcwd()
        if risk_tolerance >= 6:
            risk_message = f"{Fore.RED}All commands will be executed without confirmation.{Style.RESET_ALL}"
        elif risk_tolerance <= 0:
            risk_message = f"{Fore.GREEN}All commands will require confirmation before execution.{Style.RESET_ALL}"
        else:
            risk_message = f"{Fore.YELLOW}Commands with a risk score above {risk_tolerance} will require confirmation before execution.{Style.RESET_ALL}"
        
        print(f"{Fore.CYAN}Welcome to gptautocli! {Style.RESET_ALL} \n  Using model: {model} \n  Current directory: {current_dir}\n  Detected OS: {osType}\n  {risk_message}")
        self.model = model
    
    def riskConfirmation(self, command, risk_score):
        choice = ''
        if risk_score >= 5:
            choice = input(Fore.MAGENTA + f"Risk score of {risk_score} detected for command: {command}.  Proceed? (y/n): " + Style.RESET_ALL)
        elif risk_score >= 4:
            choice = input(Fore.RED + f"Risk score of {risk_score} detected for command: {command}.  Proceed? (y/n): " + Style.RESET_ALL)
        elif risk_score == 1:
            choice = input(Fore.GREEN + f"Risk score of {risk_score} detected for command: {command}.  Proceed? (y/n): " + Style.RESET_ALL)
        else:
            choice = input(Fore.YELLOW + f"Risk score of {risk_score} detected for command: {command}.  Proceed? (y/n): " + Style.RESET_ALL)
        return choice.lower() == "y"

    def choose_chat_history(self, history):
        return []

    def get_user_input(self):
        input_text = input(Fore.CYAN + "You: " + Style.RESET_ALL)
        # make sure the input is not empty
        while not input_text:
            print(Fore.RED + "Please enter a message." + Style.RESET_ALL)
            input_text = input(Fore.CYAN + "You: " + Style.RESET_ALL)
        return input_text

    def get_LLM_model(self):
        return self.model
    
    def error(self, message):
        print(Fore.RED + message + Style.RESET_ALL)

    def info(self, message):
        print(Fore.CYAN + message + Style.RESET_ALL)

    def chatBotMessage(self, message):

        print(Fore.GREEN + self.model + ": "+ Style.RESET_ALL + message)

    def dialog(self, message, secure=False):
        if not secure:
            return input(message + ": ")
        else:
            # use getpass to hide the input
            return getpass.getpass(message + ": ")
        
    def isInProgess(self):
        return self.inProgress
    
    def inProgressStart(self, function_name, arguments):
        self.inProgress = True
        print(f"{Fore.YELLOW}AI entering the terminal.  Enter q to stop the process and return to the chat.{Style.RESET_ALL}")

    def inProgressEnd(self):
        self.inProgress = False
        print(f"{Fore.GREEN}Command completed.{Style.RESET_ALL}")

    def command(self, input):
        print('Command: ' + input)

    def commandResult(self, output):
        # if the otput ends with a newline, remove it
        if output.endswith("\n"):
            output = output[:-1]
        print(output)


from setuptools import setup, find_packages

with open("VERSION") as f:
    version = f.read().strip()

with open("requirements.txt") as f:
    install_requires = f.read().splitlines()



setup(
    name="gptautocli", 
    version=version,
    author="Benjamin Schoolland",
    author_email="bschoolland@gmail.com",
    description="Provides a conversational interface to the terminal, allowing you to run commands and perform tasks using natural language.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/BSchoolland/gptautocli",
    packages=find_packages(), 
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'gptautocli = gptautocli.main:main',
        ],
    },
    python_requires=">=3.6",
    install_requires=install_requires,
)





import os
import platform

def get_terminal_type():
    print('platform.system():', platform.system())
    # Check for terminal types on Linux
    if platform.system() == 'Linux':
        if os.path.exists('/usr/bin/gnome-terminal'):
            return 'gnome-terminal'
        elif os.path.exists('/usr/bin/konsole'):
            return 'konsole'
        elif os.path.exists('/usr/bin/xterm'):
            return 'xterm'
        else:
            return 'unknown'
    # Check for terminal on macOS
    elif platform.system() == 'Darwin':
        if os.path.exists('/Applications/iTerm.app'):
            return 'iterm'
        elif os.path.exists('/Applications/Utilities/Terminal.app'):
            return 'mac_terminal'
        # zsh is the default shell on macOS
        elif os.path.exists('/bin/zsh'):
            return 'zsh'
        else:
            return 'unknown'
    # Return a generic terminal type for Windows
    elif platform.system() == 'Windows':
        return 'cmd'
    else:
        return 'unknown'

def is_powershell():
    # Check if the shell is PowerShell
    return 'pwsh' in os.getenv('SHELL', '').lower() or 'powershell' in os.getenv('SHELL', '').lower()

def get_os_type():
    return platform.system()